import time
import json
import jwt
import urllib
import requests
import logging
import sys

__author__ = 'Sisense'

# Logging settings
LOG_FILE_NAME = 'C:\ProgramData\Sisense\PrismServer\example.log'
LOG_LEVEL = logging.DEBUG

logging.basicConfig(filename=LOG_FILE_NAME, level=logging.DEBUG)

REST_API_TOKEN = '6014a121a88aadfeb9d6879c571f96a4'

SISENSE_BASE_URL = "http://localhost:8081"

EMAIL = 'david.hogeg@sisense.com'

PASSWORD = 'Sisense'


def create_jwt_token():
    payload = {
        "iat": int(time.time())
    }

    shared_key = REST_API_TOKEN
    payload['email'] = EMAIL
    payload['password'] = PASSWORD

    jwt_string = jwt.encode(payload, shared_key)
    encoded_jwt = urllib.quote_plus(jwt_string)  # url-encode the jwt string

    return encoded_jwt


def call_api(http_method, url_suffix, params=None, payload=None, sys_admin=False):

    encoded_jwt = create_jwt_token()
    print encoded_jwt
    headers = {'x-api-key': encoded_jwt}

    if payload:
        headers['Content-Type'] = 'application/json'

    url = SISENSE_BASE_URL + url_suffix

    response = None
    data = None

    if http_method == 'GET':
        response = requests.get(url, params=params, headers=headers)
    elif http_method == 'POST':
        data = json.dumps(payload)
        response = requests.post(
            url, data=data, params=params, headers=headers)
    elif http_method == 'PATCH':
        data = json.dumps(payload)
        response = requests.patch(
            url, data=data, params=params, headers=headers)

    return response


def get_users(params=None):

    res = call_api('GET', '/api/users', params=params)
    if res.reason == 'OK':
        return json.loads(res.text)

    return None


def get_shares(dashboard_id):

    print ('/api/v1/dashboards/{0}/shares'.format(dashboard_id))

    res = call_api('GET', '/api/v1/dashboards/{0}/shares'.format(dashboard_id))

    print res.text

    if res.reason == 'OK':
        return json.loads(res.text)

    return None

# ----------------------
# Share a dashboard with a user
# ----------------------


def share_dashboard_with_user(user_name, dashboard_id, role='view', subscribe=False):

    user_id = get_user_id(user_name)

    logging.debug("User id retrieved: " + user_id)

    if user_id is not None:
        share_dashboard(user_id, dashboard_id, 'user', role, subscribe)

# ----------------------
#  Share a dashboard with a group
# -----------------------


def share_dashboard_with_group(group_name, dashboard_id, role='view', subscribe=False):

     # Get the groupId required for updating the share
    group_id = get_group_id(group_name)

    logging.debug("Group id retrieved: " + group_id)

    if group_id is not None:
        share_dashboard(group_id, dashboard_id, 'group', role, subscribe)

# --------------------
# Generic share dashborad function to be called by
# share_dashboard_with_user and share_dashboard_with_group
# functions.
# --------------------


def share_dashboard(share_id, dashboard_id, share_type, role='view', subscribe=False):

    logging.info("Stated share_dahsboard , sharing dashboard " +
                 dashboard_id + " with " + share_type + " " + share_id)

    # Retrieve the current shares
    curr_shares = get_shares(dashboard_id)

    logging.debug("Current shares:\n" +
                  str(json.dumps(curr_shares, indent=4, separators=(',', ': '))))

    # Check share doesn't already exist
    curr_shares = [
        shares for shares in curr_shares if shares.get('shareId') != share_id]

    # Add the new share to the shares array
    curr_shares.append({
        "shareId": share_id,
        "type": share_type,
        "rule": role,
        "subscribe": subscribe
    })

    # Build the shares JSON
    curr_shares = {"shares": curr_shares}

    logging.debug("Current shares after append:\n" +
                  str(json.dumps(curr_shares, indent=4, separators=(',', ': '))))

    # Run the command
    res = call_api('PATCH', '/api/v1/dashboards/' +
                   dashboard_id, payload=curr_shares)

    logging.debug(res.text)

    if res.reason == 'OK':
        return '200'

    return None

# -------------------
# Share all dashbaords in a folder with a user
# This function simply loops over the dahsboards in
# a folder and shares each dashbpoard with a user
# -------------------


def share_folder_with_user(email, folder_name):

    logging.info("Stated share_folder_with_user , sharing folder " +
                 folder_name + " with user " + email)

    dashboard_ids_arr = get_dashboards_in_folder(folder_name)

    logging.debug("Found dashboards in folder:\n" + str(dashboard_ids_arr))

    for dashboard_id in dashboard_ids_arr:

        res = share_dashboard_with_user(email, dashboard_id)

        print ("Sharing dashboard: " + dashboard_id +
               " with " + email + ". Result: " + str(res))

    logging.info(__name__ + "complete")


def share_folder_with_group(group_name, folder_name):

    logging.info("Stated share_folder_with_group , sharing folder " +
                 folder_name + " with group " + group_name)

    dashboard_ids_arr = get_dashboards_in_folder(folder_name)

    logging.debug("Found dashboards in folder:\n" + str(dashboard_ids_arr))

    for dashboard_id in dashboard_ids_arr:

        res = share_dashboard_with_group(group_name, dashboard_id)

        print ("Sharing dashboard: " + dashboard_id +
               " with " + group_name + ". Result: " + str(res))

    logging.info(__name__ + "complete")


def get_group_id(group_name):

    res = call_api('GET', '/api/v1/groups?name=' + group_name)

    if res.status_code == 404 and res.reason == 'Not Found':
        return None

    elif res.status_code == 200:

        group_json = json.loads(res.text)
        group_json = group_json[0]  # since the GET call returns an array

        return (group_json.get('_id'))
    else:
        return None


def get_user_id(user_name):

    res = call_api('GET', '/api/users/' + user_name)

    if res.status_code == 404 and res.reason == 'Not Found':

        logging.info("User name: " + user_name + " not found.")
        return (res.status_code)

    elif res.status_code == 200:

        logging.info("Found user name: " + user_name)
        user_json = json.loads(res.text)

        logging.debug("User_json:" + str(user_json))

        return (user_json.get('_id'))
    else:
        return None

# ------------------
# Get folder ID by name
# ------------------


def get_folder_id(folder_name):

    res = call_api('GET', '/api/v1/folders?name=' + folder_name)

    if res.status_code == 404 and res.reason == 'Not Found':
        return (None)
    elif res.status_code == 200:

        folder_json = json.loads(res.text)
        folder_json = folder_json[0]  # since the GET call returns an array
        logging.debug('Folder found. Folderid=' + folder_json.get('oid'))

        return (folder_json.get('oid'))
    else:
        return (None)

#-----------------
# Find ids of dashboards in a folder, by folder name
#-----------------


def get_dashboards_in_folder(folder_name):

    folder_id = get_folder_id(folder_name)

    res = call_api('GET', '/api/v1/dashboards?parentFolder=' +
                   folder_id + '&fields=_id')

    if res.status_code == 200:

        logging.debug("Found dashboards for folder: " + res.text)

        dashboard_ids = []
        dashboard_ids = [ids['oid'] for ids in json.loads(res.text)]

        logging.debug("Dashboard Ids found in folder: " + str(dashboard_ids))

    return dashboard_ids


def main(argv):

    try:

        share_dashboard_with_user('test', '56f1aa1dbc4b62043a000028')
        # share_dashboard_with_group('Test', '56f1aa1dbc4b62043a000028')
        # share_folder_with_user("test@test.com", "JS")
        # share_folder_with_group("Test","JS")
        # get_folder_id("JS")
        # get_dashboards_in_folder("JS")

    except Exception, e:

        logging.error(e)

    else:
        print "Completed Successfully"

if __name__ == '__main__':
    main(sys.argv)
